// Macros and functions that are completely internal.

#define APD_MAX_STRING_SIZE  5000

// If APD_INTERNAL_REPORT is called with priority greater or equal to the
// apd_object->internal_debug_level, then that action will be taken
// This is the default value for apd_object->internal_debug_level
#ifndef APD_DEFAULT_INTERNAL_DEBUG_LEVEL
#    define APD_DEFAULT_INTERNAL_DEBUG_LEVEL 3
#endif

// For debugging *this* library, we can't rely on its own functions.
// Instead, use this simple macro:
#if APD_INTERNAL_DEBUG_COMPILE == 1
    #define APD_INTERNAL_REPORT(priority, ...) \
        do \
          { \
            if ( (apd_object->internal_debug) && (apd_object->internal_debug_level >= priority) ) \
              { \
                fprintf(APD_DEFAULT_ERROR_STREAM, APD_FILE_LINE_FORMAT, __FILE__, __LINE__); \
                fprintf(APD_DEFAULT_ERROR_STREAM, APD_FUNCTION_NAME_FORMAT, APD_FUNCTION_NAME); \
                fprintf(APD_DEFAULT_ERROR_STREAM, __VA_ARGS__); \
                fprintf(APD_DEFAULT_ERROR_STREAM, "\n"); \
                fflush(APD_DEFAULT_ERROR_STREAM); \
              } \
          } /* end do-while */ \
        while (0) /* swallow semicolon */
#else
    #define APD_INTERNAL_REPORT(priority, ...)
#endif

// APD internal assertions are always on all the time
#define APD_INTERNAL_ASSERT(test_expression, ...) \
    APD_ASSERT_GUTS(0, test_expression, APD_DEFAULT_ERROR_STREAM, /* do nothing */, __VA_ARGS__);

// This isn't even used currently, but it could be...
#if APD_INTERNAL_DEBUG_COMPILE == 1
    #define APD_INTERNAL_ACTION(...) \
        do \
          { \
            if (apd_object->internal_debug) \
              { \
                __VA_ARGS__ \
              } \
          } while(0) /*swallow semicolon */
#else
    #define APD_INTERNAL_ACTION(...)
#endif

#if APD_INTERNAL_DEBUG_COMPILE == 1
    #define APD_CHECK_DEBUG_OBJECT(object) if (object->check_internal) { _apd_debug_struct_check(object); }
    #define APD_BACKUP_DEBUG_OBJECT(object) if (object->check_internal) { _apd_debug_struct_backup(object); }
#else
    #define APD_CHECK_DEBUG_OBJECT(object) /* APD_CHECK_INTERNAL not defined, no debug object checking */
    #define APD_BACKUP_DEBUG_OBJECT(object) /* APD_CHECK_INTERNAL not defined, no debug object checking */
#endif

inline void _apd_check_if_function_tracking_initialized(struct apd_object_struct *apd_object_internal);
void _apd_debug_struct_backup(struct apd_object_struct *apd_object_internal);
void _apd_debug_struct_check(struct apd_object_struct *apd_object_internal);

