#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <apd.h>
#include <apd_internal.h>

// ========================================
// ========================================
// Copy main contents of debug structure to backup copy
// ========================================
void _apd_debug_struct_backup(struct apd_object_struct *apd_object_internal)
{
struct apd_function_description_struct *function_info;
struct apd_function_description_struct *function_info_backup;
int ds_index;

APD_INTERNAL_REPORT(5, "starting");

// Free old backup description stack
APD_INTERNAL_REPORT(5, "Freeing old backup description stack");
APD_INTERNAL_REPORT(5, "apd_object_internal->backup at $%lX", (unsigned long int)apd_object_internal->backup);
APD_INTERNAL_REPORT(5, "apd_object_internal->backup->description_stack_size = %d",
      apd_object_internal->backup->description_stack_size);
if (apd_object_internal->backup->description_stack_size > 0)
  {
    for (ds_index = 0; (uint)ds_index < apd_object_internal->backup->description_stack_size; ds_index++)
      {
        // Get backup function_info off linked_list stack (empty from element 0, stack-style)
        function_info_backup =
              (struct apd_function_description_struct *)g_slist_nth_data(apd_object_internal->backup->description_stack, 0);
        APD_INTERNAL_REPORT(5, "function_info_backup at $%lX", (unsigned long int)function_info_backup);
        // Remove function_info element from backup description stack
        APD_INTERNAL_REPORT(5, "Removing function_info_backup from list");
        apd_object_internal->backup->description_stack =
              g_slist_remove( apd_object_internal->backup->description_stack, (gpointer)function_info_backup );
        APD_INTERNAL_REPORT(5, "function_info_backup->importance = %d", function_info_backup->importance);
        // Free name and description
        APD_INTERNAL_REPORT(5, "freeing function_info_backup->name at $%lX", (unsigned long int)function_info_backup->name);
        free(function_info_backup->name);
        APD_INTERNAL_REPORT(5, "freeing function_info_backup->description at $%lX", (unsigned long int)function_info_backup->description);
        free(function_info_backup->description);
        // Free function_info backup element
        APD_INTERNAL_REPORT(5, "freeing function_info_backup at $%lX", (unsigned long int)function_info_backup);
        free(function_info_backup);
      } // end for ds_index
  } // end if stack size > 0

// Copy simple elements (wait until after old backup->description_stack_size not needed any more)
APD_INTERNAL_REPORT(5, "copying simple elements from main to backup");
apd_object_internal->backup->report_threshold = apd_object_internal->report_threshold;
apd_object_internal->backup->assert_threshold = apd_object_internal->assert_threshold;
apd_object_internal->backup->max_function_depth = apd_object_internal->max_function_depth;
apd_object_internal->backup->description_stack_size = apd_object_internal->description_stack_size;
apd_object_internal->backup->internal_debug = apd_object_internal->internal_debug;
apd_object_internal->backup->internal_debug_level = apd_object_internal->internal_debug_level;
apd_object_internal->backup->error_stream = apd_object_internal->error_stream;

// Copy description stack
APD_INTERNAL_REPORT(5, "apd_object_internal at $%lX", (unsigned long int)apd_object_internal);
APD_INTERNAL_REPORT(5, "apd_object_internal->backup at $%lX", (unsigned long int)apd_object_internal->backup);
APD_INTERNAL_REPORT(5, "apd_object_internal->description_stack_size = %d",
      apd_object_internal->description_stack_size);
if (apd_object_internal->description_stack_size > 0)
  {
    for (ds_index = apd_object_internal->description_stack_size-1; ds_index >= 0; ds_index--)
      {
        // Get real function_info off linked_list stack
        function_info = (struct apd_function_description_struct *)
                        g_slist_nth_data(apd_object_internal->description_stack, (uint)ds_index);
        APD_INTERNAL_REPORT(5, "read function_info at $%lX", (unsigned long int)function_info);
        // Allocate backup function_info
        function_info_backup = malloc(sizeof *function_info_backup);
        APD_INTERNAL_REPORT(5, "allocated function_info_backup at $%lX", (unsigned long int)function_info_backup);
        APD_INTERNAL_ASSERT( function_info_backup != NULL, "error allocating memory for function_info");
        // Copy elements from function_info to backup
        function_info_backup->importance = function_info->importance;
        // Allocate and copy name
        function_info_backup->name = malloc(strlen(function_info->name)+1);
        APD_INTERNAL_REPORT(5, "allocating function_info_backup->name at $%lX", (unsigned long int)function_info_backup->name);
        APD_INTERNAL_ASSERT( function_info_backup->name != NULL, "");
        strcpy(function_info_backup->name, function_info->name);
        // Allocate and copy description
        function_info_backup->description = malloc(strlen(function_info->description)+1);
        APD_INTERNAL_REPORT(5, "allocating function_info_backup->description at $%lX", (unsigned long int)function_info_backup->description);
        APD_INTERNAL_ASSERT( function_info_backup->description != NULL, "");
        strcpy(function_info_backup->description, function_info->description);
        // Insert new function_info_backup into backup description
        APD_INTERNAL_REPORT(5, "inserting function_info_backup into backup description list");
        apd_object_internal->backup->description_stack =
              g_slist_insert(apd_object_internal->backup->description_stack, (gpointer)function_info_backup, 0);
      } // end for ds_index
  } // end if stack size > 0
APD_INTERNAL_REPORT(5, "ending");
} // end _apd_debug_struct_backup()



// ========================================
// ========================================
// Check the apd_object to make sure the backup and the
// main contents are the same to catch memory corruption
// ========================================
void _apd_debug_struct_check(struct apd_object_struct *apd_object_internal)
{
struct apd_function_description_struct *function_info;
struct apd_function_description_struct *function_info_backup;
int ds_index;

APD_INTERNAL_REPORT(5, "starting");

// Check simple elements
APD_INTERNAL_REPORT(5, "Checking simple elements");
APD_INTERNAL_ASSERT( apd_object_internal->backup->report_threshold == apd_object_internal->report_threshold,
      "apd memory backup does not match main: probable apd memory corruption");
APD_INTERNAL_ASSERT(apd_object_internal->backup->assert_threshold == apd_object_internal->assert_threshold,
      "apd memory backup does not match main: probable apd memory corruption");
APD_INTERNAL_ASSERT(apd_object_internal->backup->max_function_depth == apd_object_internal->max_function_depth,
      "apd memory backup does not match main: probable apd memory corruption");
APD_INTERNAL_ASSERT(apd_object_internal->backup->description_stack_size == apd_object_internal->description_stack_size,
      "apd memory backup does not match main: probable apd memory corruption");
APD_INTERNAL_ASSERT(apd_object_internal->backup->internal_debug == apd_object_internal->internal_debug,
      "apd memory backup does not match main: probable apd memory corruption");
APD_INTERNAL_ASSERT(apd_object_internal->backup->internal_debug_level == apd_object_internal->internal_debug_level,
      "apd memory backup does not match main: probable apd memory corruption");
APD_INTERNAL_ASSERT((unsigned long int)(apd_object_internal->backup->error_stream)
      == (unsigned long int)(apd_object_internal->error_stream),
      "apd memory backup does not match main: probable apd memory corruption");

// Check description stack
APD_INTERNAL_REPORT(5, "Checking description stack");
APD_INTERNAL_REPORT(5, "apd_object_internal at $%lX", (unsigned long int)apd_object_internal);
APD_INTERNAL_REPORT(5, "apd_object_internal->backup at $%lX", (unsigned long int)apd_object_internal->backup);
APD_INTERNAL_REPORT(5, "apd_object_internal->description_stack_size = %d",
      apd_object_internal->description_stack_size);
if (apd_object_internal->description_stack_size > 0)
  {
    for (ds_index = 0; (uint)ds_index < apd_object_internal->description_stack_size; ds_index++)
      {
        APD_INTERNAL_REPORT(5, "Checking description stack element %d", ds_index);
        // Get function_info's off linked_list stack
        function_info =
              (struct apd_function_description_struct *)
              g_slist_nth_data(apd_object_internal->description_stack, (uint)ds_index);
        APD_INTERNAL_REPORT(5, "Checking main description stack function_info at $%lX",
                (unsigned long int)function_info);
        function_info_backup =
              (struct apd_function_description_struct *)
              g_slist_nth_data(apd_object_internal->backup->description_stack, (uint)ds_index);
        APD_INTERNAL_REPORT(5, "Checking backup description stack function_info_backup at $%lX",
                (unsigned long int)function_info_backup);

        // Compare thresholds
        APD_INTERNAL_REPORT(5, "compare function importance");
        APD_INTERNAL_ASSERT(function_info->importance == function_info_backup->importance,
            "apd memory backup does not match main: probable apd memory corruption");
        // Compare name and description
        APD_INTERNAL_REPORT(5, "compare name and description");
        APD_INTERNAL_REPORT(5, "Checking name: element %d: backup at $%lX, main at $%lX",
                ds_index, (unsigned long int)function_info_backup->name, 
                (unsigned long int)function_info->name);
        APD_INTERNAL_ASSERT(strcmp(function_info->name, function_info_backup->name) == 0,
            "apd memory backup does not match main: probable apd memory corruption");
        APD_INTERNAL_REPORT(5, "Checking description: element %d: backup at $%lX, main at $%lX",
                ds_index, (unsigned long int)function_info_backup->description,
                (unsigned long int)function_info->description);
        APD_INTERNAL_ASSERT(strcmp(function_info->description, function_info_backup->description) == 0,
            "apd memory backup does not match main: probable apd memory corruption");
      } // end for ds_index
  } // end if stack size > 0

APD_INTERNAL_REPORT(5, "ending");
} // end _apd_debug_struct_check()
