#include <apd.h>
#include <apd_internal.h>

void _apd_pop_call_description(struct apd_object_struct *apd_object_internal, char *file, int line, const char *function_name)
{
struct apd_function_description_struct *function_info;
char *function_name_pointer;

APD_INTERNAL_REPORT(5, "starting");

_apd_check_if_function_tracking_initialized(apd_object_internal);

// Check to make sure main apd_object contents still match backup copy like they should
APD_CHECK_DEBUG_OBJECT(apd_object_internal);
APD_INTERNAL_REPORT(5, "apd_object_internal at $%lX", (unsigned long int)apd_object_internal);

// Check to make sure APD_FUNCTION_NAME was defined
if (strcmp(function_name, "!") == 0) APD_DIE("fatal error: APD_FUNCTION_NAME not defined");

// Get function_info item from description stack
APD_INTERNAL_REPORT(5, "description_stack at $%lX", (unsigned long int)apd_object_internal->description_stack);
function_info = (struct apd_function_description_struct *)g_slist_nth_data(apd_object_internal->description_stack, 0);
APD_INTERNAL_REPORT(5, "function_info at $%lX", (unsigned long int)function_info);
APD_INTERNAL_REPORT(5, "function_info->name at $%lX", (unsigned long int)function_info->name);
APD_INTERNAL_REPORT(5, "function_info->description at $%lX", (unsigned long int)function_info->description);
APD_INTERNAL_REPORT(5, "description_stack at $%lX", (unsigned long int)apd_object_internal->description_stack);
APD_INTERNAL_ASSERT(function_info != NULL, "Function description stack is empty");
APD_INTERNAL_REPORT(2, "poping function: %s", function_info->name);
apd_object_internal->description_stack = g_slist_remove( apd_object_internal->description_stack, (gpointer)function_info );
APD_INTERNAL_ASSERT(function_info != NULL, "Trying to pop a NULL description");

// Get a pointer to the function name from function_info past the level_spacers
function_name_pointer = function_info->name;
while (*function_name_pointer == APD_LEVEL_SPACER_CHARACTER) function_name_pointer++;

// Check to make sure function_name is the same as the one PUSHed
APD_INTERNAL_ASSERT(strcmp(function_name, function_name_pointer) == 0,
      "function name being popped (%s) does not match the function name at top of stack (%s)\n"
      "There is probably a APD_PUSH_FUNCTION() without a corresponding APD_POP_FUNCTION()",
      function_name, function_name_pointer);

// Report if needed
if (apd_object_internal->report_threshold >= function_info->importance)
  {
    fprintf(apd_object_internal->error_stream, APD_FUNCTION_NAME_FORMAT " (end) }}}%d", 
            function_info->name, apd_object_internal->description_stack_size);
    fprintf(apd_object_internal->error_stream, APD_BEFORE_FILE_LINE_FORMAT APD_FILE_LINE_FORMAT "\n", file, line);
    fflush(apd_object_internal->error_stream);
  }

// Pop the 0 element off the description list
APD_INTERNAL_REPORT(5, "function_info->description_stack_size = %d", apd_object_internal->description_stack_size);
APD_INTERNAL_ASSERT(apd_object_internal->description_stack_size > 0, "attempt to pop an empty description stack");
APD_INTERNAL_REPORT(5, "freeing function_info->description at $%lX", (unsigned long int)function_info->description);
free( function_info->description );
APD_INTERNAL_REPORT(5, "freeing function_info->name");
free( function_info->name );
APD_INTERNAL_REPORT(5, "freeing function_info");
free( function_info);
apd_object_internal->description_stack_size--;
APD_INTERNAL_REPORT(5, "function_info->description_stack_size = %d", apd_object_internal->description_stack_size);

// Update current function name
if (apd_object_internal->description_stack_size == 0)
  {
    apd_object_internal->current_function_name = NULL;
  }
else
  {
    function_info = (struct apd_function_description_struct *)g_slist_nth_data(apd_object_internal->description_stack, 0);
    APD_INTERNAL_REPORT(5, "function_info at $%lX", (unsigned long int)function_info);
    APD_INTERNAL_REPORT(5, "function_info->name at $%lX", (unsigned long int)function_info->name);
    apd_object_internal->current_function_name = function_info->name;
  }

// Increase report_threshold if this option is set
if (apd_object_internal->report_threshold_decreases_with_stack == 1) apd_object_internal->report_threshold++;

// Refresh apd_object's backup copy since we just changed it
APD_BACKUP_DEBUG_OBJECT(apd_object_internal);
APD_INTERNAL_REPORT(5, "ending");
} // end _apd_pop_call_description

